﻿namespace Tenma.HomeMatic.Remoting
{
    using System.Collections;
    using System.Collections.Generic;

    /// <summary>
    /// Definiert den Proxy für die
    /// Kommunikation mit dem BidCoS-Service oder der
    /// HomeMatic Zentrale.
    /// </summary>
    public interface IRpcProxy
    {
        /// <summary>
        /// Liefert oder setzt die Url des BidCoS-Services.
        /// </summary>
        string Url { get; set; }

        /// <summary>
        /// Liefert oder setzt den Timeout des BidCoS-Services.
        /// </summary>
        int Timeout { get; set; }

        /// <summary>
        /// Aktiviert eine direkte Geräteverknüpfung.
        /// </summary>
        /// <param name="address">Seriennummer eines Kanals</param>
        /// <param name="peerAddress">Seriennummer des Verknüpfungspartners</param>
        /// <param name="longPress">true, um den langen Tastendruck auszuwählen</param>
        void ActivateLinkParamset(string address, string peerAddress, bool longPress);

        /// <summary>
        /// Fügt ein Gerät anhand seiner Seriennummer hinzu.
        /// </summary>
        /// <param name="serialNumber">Seriennummer des Geräts</param>
        /// <returns>beschreibung des Geräts</returns>
        IDeviceDescription AddDevice(string serialNumber);

        /// <summary>
        /// Erstellt eine direkte Geräteverknüpfung.
        /// </summary>
        /// <param name="sender">Seriennummer des Senders</param>
        /// <param name="receiver">Seriennummer des Empfängers</param>
        /// <param name="name">Name der Verknüpfung</param>
        /// <param name="description">Beschreibung der Verknüpfung</param>
        void AddLink(string sender, string receiver, string name, string description);

        /// <summary>
        /// Ändert den System-Sicherheitsschlüssel.
        /// </summary>
        /// <param name="passphrase">Neuer System-Sicherheitsschlüssel</param>
        void ChangeKey(string passphrase);

        /// <summary>
        /// Löscht die im BidCoS-Service zwischengespeicherten
        /// Konfigurationsdaten für ein Gerät.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        void ClearConfigCache(string address);

        /// <summary>
        /// Löscht ein Gerät.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <param name="flags">Flags für das Löschen</param>
        void DeleteDevice(string address, int flags);

        /// <summary>
        /// Ermittelt den Wert eines Parameters.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <param name="paramsetKey">Schlüssel des Paramsets</param>
        /// <param name="parameterId">Id des Parameters</param>
        void DetermineParameter(string address, string paramsetKey, string parameterId);

        /// <summary>
        /// Liefert die Beschreibung eines Geräts.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <returns>Beschreibung des Geräts</returns>
        IDeviceDescription GetDeviceDescription(string address);

        /// <summary>
        /// Liefert die Zeit in Sekunden,
        /// für die der Anlernmodus noch aktiv ist.
        /// </summary>
        /// <returns>
        /// Anzahl der Sekunden, für die der
        /// Anlernmodus noch aktiv ist.
        /// </returns>
        int GetInstallMode();

        /// <summary>
        /// Liefert die Seriennummer des letzten Geräts,
        /// bei dem die Kommunikation aufgrund von
        /// Authentifizierungsproblemen fehlgeschlagen ist.
        /// </summary>
        /// <param name="reset">Setzt den Wert zurück</param>
        /// <returns>
        /// Seriennummer des letzt Geräts, bei dem die
        /// Authentifizierung fehlgeschlagen ist.
        /// </returns>
        string GetKeyMissmatchDevice(bool reset);

        /// <summary>
        /// Liefert Informationen zu einer Verknüpfung.
        /// </summary>
        /// <param name="sender">Seriennummer des Senders</param>
        /// <param name="receiver">Seriennummer des Empfängers</param>
        /// <returns>Name und Beschreibung der Verknüpfung</returns>
        string[] GetLinkInfo(string sender, string receiver);

        /// <summary>
        /// Liefert die Verknüpfungspartner eines Geräts.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <returns>Seriennummern der Verknüpfungspartner</returns>
        string[] GetLinkPeers(string address);

        /// <summary>
        /// Liefert die Beschreibungen von direkten
        /// Verknüpfungen.
        /// </summary>
        /// <param name="address">Adresse eines Geräts</param>
        /// <param name="flags">Flags für die Abfrage</param>
        /// <returns>Beschreibungen der Verknüpfungen</returns>
        ILinkDescription[] GetLinks(string address, int flags);

        /// <summary>
        /// Liefert éin Paramset.
        /// </summary>
        /// <param name="address">Adresse des Geräts</param>
        /// <param name="paramsetKey">Schlüssel des Paramsets</param>
        /// <returns>Gelesenes Paramset</returns>
        Hashtable GetParamset(string address, string paramsetKey);

        /// <summary>
        /// Liefert die Beschreibung eines Paramsets.
        /// </summary>
        /// <param name="address">Adresse des Geräts</param>
        /// <param name="type">Typ des Paramsets</param>
        /// <returns>Beschreibung des Paramsets</returns>
        Dictionary<string, IParameterDescription> GetParamsetDescription(string address, string type);

        /// <summary>
        /// Liefert die Id eines Paramsets.
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <param name="type">Typ des Paramsets</param>
        /// <returns>Id des Paramsets</returns>
        string GetParamsetId(string address, string type);

        /// <summary>
        /// Liefert einen Wert aus dem VALUES-Paramset
        /// </summary>
        /// <param name="address">Seriennummer des Geräts</param>
        /// <param name="valueKey">Schlüssel des Parameters</param>
        /// <returns>Gelesener Wert</returns>
        object GetValue(string address, string valueKey);

        /// <summary>
        /// Meldet einen XML-RPC-Server für die
        /// Ereignisbehandlung an.
        /// </summary>
        /// <param name="url">Url des Servers</param>
        /// <param name="interfaceId">
        /// Id welche der BidCoS-Service bei Anfragen
        /// an den BidCoS-Service verwendet.
        /// </param>
        void Init(string url, string interfaceId);

        /// <summary>
        /// Meldet einen XML-RPC-Server für die
        /// Ereignisbehandlung ab.
        /// </summary>
        /// <param name="url">Url des Servers</param>
        void Shutdown(string url);

        IDeviceDescription[] ListDevices();

        IDeviceDescription[] listTeams();

        int GetLogLevel();

        int SetLogLevel(int logLevel);

        void PutParamset(string addres, string paramsetKey, Hashtable paramset);

        void RemoveLink(string sender, string receiver);

        bool ReportValueUsage(string address, string valueId, int refCounter);

        void RestoreConfigToDevice(string address);

        RssiInfo[] GetRssiInfo();

        int SearchDevices();

        void SetInstallMode(bool isOn);

        void SetLinkInfo(string sender, string receiver, string name, string description);

        void SetTeam(string channelAddress, string teamAddress);

        void SetTempKey(string passphrase);

        void SetValue(string address, string valueKey, object value);

        bool[] UpdateFirmware(string[] deviceAddresses);

        IBidcosInterface[] ListBidcosInterfaces();

        void SetBidCosIntrface(string devieAddress, string interfaceAddress, bool useRoaming);

        IServiceMessage[] GetServiceMessages();

        object GetMetadata(string objectId, string dataId);

        void SetMetadata(string objectId, string dataId, object value);

        Hashtable GetAllMetadata(string objectId);
    }
}
